@@ -25,7 +25,8 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-desktop-thumbnail.h>
+#include <string.h>
 
 #include "cheese-fileutil.h"
 #include "eog-thumbnail.h"
@@ -45,7 +46,7 @@ typedef struct
   CheeseFileUtil *fileutil;
   GFileMonitor   *photo_file_monitor;
   GFileMonitor   *video_file_monitor;
-  GnomeThumbnailFactory *factory;
+  GnomeDesktopThumbnailFactory *factory;
   gboolean multiplex_thumbnail_generator;
 } CheeseThumbViewPrivate;
 
@@ -82,16 +83,16 @@ cheese_thumb_view_thread_append_item (gpointer data)
   CheeseThumbView           *thumb_view = item->thumb_view;
   CheeseThumbViewPrivate    *priv       = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
 
-  GnomeThumbnailFactory *factory = priv->factory;
-  GFile                 *file    = item->file;
-  GtkTreeIter            iter    = item->iter;
-  GdkPixbuf             *pixbuf  = NULL;
-  GFileInfo             *info;
-  char                  *thumb_loc;
-  GTimeVal               mtime;
-  char                  *mime_type;
-  char                  *uri;
-  char                  *filename;
+  GnomeDesktopThumbnailFactory *factory = priv->factory;
+  GFile                        *file    = item->file;
+  GtkTreeIter                   iter    = item->iter;
+  GdkPixbuf                    *pixbuf  = NULL;
+  GFileInfo                    *info;
+  char                         *thumb_loc;
+  GTimeVal                      mtime;
+  char                         *mime_type;
+  char                         *uri;
+  char                         *filename;
 
   info = g_file_query_info (file, "standard::content-type,time::modified", 0, NULL, NULL);
 
@@ -106,18 +107,18 @@ cheese_thumb_view_thread_append_item (gpointer data)
   uri      = g_file_get_uri (file);
   filename = g_file_get_path (file);
 
-  thumb_loc = gnome_thumbnail_factory_lookup (factory, uri, mtime.tv_sec);
+  thumb_loc = gnome_desktop_thumbnail_factory_lookup (factory, uri, mtime.tv_sec);
 
   if (!thumb_loc)
   {
-    pixbuf = gnome_thumbnail_factory_generate_thumbnail (factory, uri, mime_type);
+    pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (factory, uri, mime_type);
     if (!pixbuf)
     {
       g_warning ("could not generate thumbnail for %s (%s)\n", filename, mime_type);
     }
     else
     {
-      gnome_thumbnail_factory_save_thumbnail (factory, pixbuf, uri, mtime.tv_sec);
+      gnome_desktop_thumbnail_factory_save_thumbnail (factory, pixbuf, uri, mtime.tv_sec);
     }
   }
   else
@@ -164,6 +165,7 @@ cheese_thumb_view_thread_append_item (gpointer data)
   g_free (mime_type);
   g_free (filename);
   g_object_unref (pixbuf);
+  g_object_unref (file);
   g_free (item);
 }
 
@@ -176,28 +178,62 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
   GtkIconTheme *icon_theme;
   GdkPixbuf    *pixbuf = NULL;
   GtkTreePath  *path;
-  char         *filename, *basename;
+  char         *filename, *basename, *col_filename;
   GError       *error = NULL;
+  gboolean      skip  = FALSE;
 
   CheeseThumbViewThreadData *data;
 
-  data             = g_new0 (CheeseThumbViewThreadData, 1);
-  data->thumb_view = g_object_ref (thumb_view);
-  data->file       = g_object_ref (file);
+  filename = g_file_get_path (file);
+
+  if (!(g_str_has_suffix (filename, PHOTO_NAME_SUFFIX)) && !(g_str_has_suffix (filename, VIDEO_NAME_SUFFIX)))
+  {
+    g_free (filename);
+    return;
+  }
+
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
+  {
+    /* check if the selected item is the first, else go through the store */
+    gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, THUMBNAIL_URL_COLUMN, &col_filename, -1);
+    if (g_ascii_strcasecmp (col_filename, filename))
+    {
+      while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter))
+      {
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, THUMBNAIL_URL_COLUMN, &col_filename, -1);
+        if (!g_ascii_strcasecmp (col_filename, filename))
+        {
+          skip = TRUE;
+          break;
+        }
+      }
+    }
+    else
+    {
+      skip = TRUE;
+    }
+    g_free (col_filename);
+    g_free (filename);
+
+    if (skip) return;
+  }
 
   if (priv->multiplex_thumbnail_generator)
   {
     char *f;
 
-    /* f = g_strdup_printf ("%s/pixmaps/cheese-%i.svg", PACKAGE_DATADIR, g_random_int_range (1, 3)); */
-    f      = g_strdup_printf ("%s/pixmaps/cheese-1.svg", PACKAGE_DATADIR);
+    f      = g_strdup_printf ("%s/pixmaps/cheese-%i.svg", PACKAGE_DATADIR, g_random_int_range (1, 4));
     pixbuf = gdk_pixbuf_new_from_file (f, NULL);
     g_free (f);
   }
   else
   {
     icon_theme = gtk_icon_theme_get_default ();
-    pixbuf     = gtk_icon_theme_load_icon (icon_theme, "image-loading", 96, 0, &error);
+    pixbuf     = gtk_icon_theme_load_icon (icon_theme,
+                                           "image-loading",
+                                           96,
+                                           GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                           &error);
   }
 
   if (!pixbuf)
@@ -205,14 +241,12 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
     g_warning ("Couldn't load icon: %s", error->message);
     g_error_free (error);
     error = NULL;
-    return;
   }
 
   filename = g_file_get_path (file);
   basename = g_path_get_basename (filename);
 
   gtk_list_store_append (priv->store, &iter);
-  data->iter = iter;
   gtk_list_store_set (priv->store, &iter,
                       THUMBNAIL_PIXBUF_COLUMN, pixbuf,
                       THUMBNAIL_URL_COLUMN, filename,
@@ -223,10 +257,15 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
   gtk_icon_view_scroll_to_path (GTK_ICON_VIEW (thumb_view), path,
                                 TRUE, 1.0, 0.5);
 
-  g_object_unref (pixbuf);
+  if (pixbuf) g_object_unref (pixbuf);
 
   if (!priv->multiplex_thumbnail_generator)
   {
+    data             = g_new0 (CheeseThumbViewThreadData, 1);
+    data->thumb_view = g_object_ref (thumb_view);
+    data->file       = g_object_ref (file);
+    data->iter       = iter;
+
     if (!g_thread_create ((GThreadFunc) cheese_thumb_view_thread_append_item,
                           data, FALSE, &error))
     {
@@ -320,23 +359,21 @@ cheese_thumb_view_on_drag_data_get_cb (GtkIconView      *thumb_view,
                                        guint             time,
                                        gpointer          user_data)
 {
-  GList        *list;
-  GtkTreePath  *tree_path = NULL;
+  GList        *list, *l;
   GtkTreeIter   iter;
   GtkTreeModel *model;
   char         *str;
   char         *uris = NULL;
   char         *tmp_str;
-  gint          i;
 
   list  = gtk_icon_view_get_selected_items (thumb_view);
   model = gtk_icon_view_get_model (thumb_view);
 
-  for (i = 0; i < g_list_length (list); i++)
+  for (l = list; l != NULL; l = l->next)
   {
-    tree_path = g_list_nth_data (list, i);
-    gtk_tree_model_get_iter (model, &iter, tree_path);
+    gtk_tree_model_get_iter (model, &iter, l->data);
     gtk_tree_model_get (model, &iter, 1, &str, -1);
+    gtk_tree_path_free (l->data);
 
     /* we always store local paths in the model, but DnD
      * needs URIs, so we must add file:// to the path.
@@ -356,8 +393,8 @@ cheese_thumb_view_on_drag_data_get_cb (GtkIconView      *thumb_view,
 
     g_free (str);
   }
-  gtk_selection_data_set (data, data->target, 8,
-                          (guchar *) uris, strlen (uris));
+  gtk_selection_data_set (data, gtk_selection_data_get_target (data),
+                          8, (guchar *) uris, strlen (uris));
   g_free (uris);
   g_list_free (list);
 }
@@ -549,7 +586,7 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
   g_mkdir_with_parents (path_videos, 0775);
   g_mkdir_with_parents (path_photos, 0775);
 
-  priv->factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+  priv->factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 
   /* connect signal to video path */
   file                     = g_file_new_for_path (path_videos);
