@@ -39,6 +39,10 @@
 #include <gtk/gtk.h>
 #include <libebook/e-book.h>
 
+#ifdef HAVE_MMKEYS
+  #include <X11/XF86keysym.h>
+#endif /* HAVE_MMKEYS */
+
 #ifdef HILDON
   #include <hildon/hildon-program.h>
 #endif
@@ -52,7 +56,6 @@
 #include "cheese-window.h"
 #include "ephy-spinner.h"
 #include "gst-audio-play.h"
-#include "gedit-message-area.h"
 #include "cheese-no-camera.h"
 #include "cheese-prefs-dialog.h"
 #include "cheese-flash.h"
@@ -61,8 +64,8 @@
 #define FULLSCREEN_POPUP_HEIGHT    40
 #define FULLSCREEN_TIMEOUT         5 * 1000
 #define FULLSCREEN_EFFECTS_TIMEOUT 15
-#define DEFAULT_WINDOW_WIDTH 600
-#define DEFAULT_WINDOW_HEIGHT 450
+#define DEFAULT_WINDOW_WIDTH       600
+#define DEFAULT_WINDOW_HEIGHT      450
 
 typedef enum
 {
@@ -119,8 +122,8 @@ typedef struct
   GtkWidget *countdown_frame_fullscreen;
   GtkWidget *countdown;
   GtkWidget *countdown_fullscreen;
-  GtkWidget *message_area_frame;
-  GtkWidget *message_area;
+  GtkWidget *info_bar_frame;
+  GtkWidget *info_bar;
 
   GtkWidget *button_effects;
   GtkWidget *button_photo;
@@ -178,65 +181,19 @@ typedef struct
   CheeseFlash *flash;
 } CheeseWindow;
 
+static void cheese_window_action_button_clicked_cb (GtkWidget *widget, CheeseWindow *cheese_window);
+
 void
 cheese_window_bring_to_front (gpointer data)
 {
   CheeseWindow *cheese_window     = data;
-  guint32       startup_timestamp = gdk_x11_get_server_time (GTK_WIDGET (cheese_window->window)->window);
+  guint32       startup_timestamp = gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (cheese_window->window)));
 
-  gdk_x11_window_set_user_time (GTK_WIDGET (cheese_window->window)->window, startup_timestamp);
+  gdk_x11_window_set_user_time (gtk_widget_get_window (GTK_WIDGET (cheese_window->window)), startup_timestamp);
 
   gtk_window_present (GTK_WINDOW (cheese_window->window));
 }
 
-/* Make url in about dialog clickable */
-static void
-cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointer data)
-{
-  GError    *error = NULL;
-  GtkWidget *error_dialog;
-  GdkScreen *screen;
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
-  gtk_show_uri (screen, url, gtk_get_current_event_time (), &error);
-  if (error != NULL)
-  {
-    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                           GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                           _("Failed to open browser to show:\n%s"), url);
-    gtk_dialog_run (GTK_DIALOG (error_dialog));
-    gtk_widget_destroy (error_dialog);
-    g_error_free (error);
-  }
-}
-
-/* Make email in about dialog clickable */
-static void
-cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpointer data)
-{
-  char      *uri;
-  GError    *error = NULL;
-  GtkWidget *error_dialog;
-  GdkScreen *screen;
-
-  uri = g_strconcat ("mailto:", email, NULL);
-
-  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
-  gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
-  if (error != NULL)
-  {
-    error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
-                                           GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                           _("Failed to open email client to send message to:\n%s"), email);
-    gtk_dialog_run (GTK_DIALOG (error_dialog));
-    gtk_widget_destroy (error_dialog);
-    g_error_free (error);
-  }
-  g_free (uri);
-}
-
 static char *
 audio_play_get_filename (CheeseWindow *cheese_window)
 {
@@ -260,6 +217,36 @@ cheese_window_delete_event_cb (GtkWidget *widget, GdkEvent event, gpointer data)
   return FALSE;
 }
 
+static gboolean
+cheese_window_key_press_event_cb (GtkWidget *win, GdkEventKey *event, CheeseWindow *cheese_window)
+{
+#ifndef HAVE_MMKEYS
+  return FALSE;
+#else
+  /* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
+   * of Mod3 to Mod5 (Mod2 is num-lock...) are pressed, we
+   * let Gtk+ handle the key */
+  if (event->state != 0
+      && ((event->state & GDK_CONTROL_MASK)
+          || (event->state & GDK_MOD1_MASK)
+          || (event->state & GDK_MOD3_MASK)
+          || (event->state & GDK_MOD4_MASK)
+          || (event->state & GDK_MOD5_MASK)))
+    return FALSE;
+
+  switch (event->keyval)
+  {
+    case XF86XK_WebCam:
+
+      /* do stuff */
+      cheese_window_action_button_clicked_cb (NULL, cheese_window);
+      return TRUE;
+  }
+
+  return FALSE;
+#endif /* !HAVE_MMKEYS */
+}
+
 static void
 cheese_window_fullscreen_clear_timeout (CheeseWindow *cheese_window)
 {
@@ -290,7 +277,7 @@ cheese_window_fullscreen_set_timeout (CheeseWindow *cheese_window)
   GSource *source;
 
   cheese_window_fullscreen_clear_timeout (cheese_window);
-  
+
   source = g_timeout_source_new (FULLSCREEN_TIMEOUT);
 
   g_source_set_callback (source, cheese_window_fullscreen_timeout_cb, cheese_window, NULL);
@@ -324,8 +311,8 @@ cheese_window_fullscreen_motion_notify_cb (GtkWidget      *widget,
     {
       cheese_window_fullscreen_show_bar (cheese_window);
     }
-    
-    //don't set the timeout in effect-chooser mode
+
+    /* don't set the timeout in effect-chooser mode */
     if (gtk_notebook_get_current_page (GTK_NOTEBOOK (cheese_window->notebook)) != 1)
       cheese_window_fullscreen_set_timeout (cheese_window);
   }
@@ -359,11 +346,11 @@ cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
                       cheese_window);
 
     gtk_window_fullscreen (GTK_WINDOW (cheese_window->window));
-    
+
     gtk_widget_set_size_request (cheese_window->effect_alignment, -1, FULLSCREEN_POPUP_HEIGHT);
     cheese_window_fullscreen_show_bar (cheese_window);
-    
-    //don't set the timeout in effect-chooser mode
+
+    /* don't set the timeout in effect-chooser mode */
     if (gtk_notebook_get_current_page (GTK_NOTEBOOK (cheese_window->notebook)) != 1)
       cheese_window_fullscreen_set_timeout (cheese_window);
 
@@ -374,7 +361,7 @@ cheese_window_toggle_fullscreen (GtkWidget *widget, CheeseWindow *cheese_window)
     gtk_widget_show_all (cheese_window->window);
     gtk_widget_hide_all (cheese_window->fullscreen_popup);
     gtk_widget_modify_bg (cheese_window->window, GTK_STATE_NORMAL, NULL);
-    
+
     g_signal_handlers_disconnect_by_func (cheese_window->window,
                                           (gpointer) cheese_window_fullscreen_motion_notify_cb, cheese_window);
     g_signal_handlers_disconnect_by_func (cheese_window->screen,
@@ -415,11 +402,11 @@ cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
   gdk_threads_leave ();
 }
 
-
 static void
 cheese_window_video_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
   gdk_threads_enter ();
+
   /* TODO look at this g_free */
   g_free (cheese_window->video_filename);
   cheese_window->video_filename = NULL;
@@ -942,7 +929,7 @@ cheese_window_cmd_command_line (GtkAction *action, CheeseWindow *cheese_window)
 static void
 cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
 {
-  GError  *error = NULL;
+  GError    *error = NULL;
   GdkScreen *screen;
 
   screen = gtk_widget_get_screen (GTK_WIDGET (cheese_window));
@@ -1202,7 +1189,7 @@ cheese_window_countdown_picture_cb (gpointer data)
   {
     cheese_flash_fire (cheese_window->flash);
     shutter_filename = audio_play_get_filename (cheese_window);
-    audio_play = gst_audio_play_file (shutter_filename, &error);
+    audio_play       = gst_audio_play_file (shutter_filename, &error);
     if (!audio_play)
     {
       g_warning ("%s", error ? error->message : "Unknown error");
@@ -1214,7 +1201,7 @@ cheese_window_countdown_picture_cb (gpointer data)
 }
 
 static void
-cheese_window_no_camera_message_area_response (GtkWidget *widget, gint response_id, GtkWidget *cheese_window)
+cheese_window_no_camera_info_bar_response (GtkWidget *widget, gint response_id, CheeseWindow *cheese_window)
 {
   GError  *error = NULL;
   gboolean ret;
@@ -1529,22 +1516,22 @@ cheese_window_radio_action_group_new (CheeseWindow *cheese_window, char *name,
 }
 
 static void
-cheese_window_set_message_area (CheeseWindow *cheese_window,
-                                GtkWidget    *message_area)
+cheese_window_set_info_bar (CheeseWindow *cheese_window,
+                                GtkWidget    *info_bar)
 {
-  if (cheese_window->message_area == message_area)
+  if (cheese_window->info_bar == info_bar)
     return;
 
-  if (cheese_window->message_area != NULL)
-    gtk_widget_destroy (cheese_window->message_area);
+  if (cheese_window->info_bar != NULL)
+    gtk_widget_destroy (cheese_window->info_bar);
 
-  cheese_window->message_area = message_area;
+  cheese_window->info_bar = info_bar;
 
-  if (message_area == NULL)
+  if (info_bar == NULL)
     return;
 
-  gtk_container_add (GTK_CONTAINER (cheese_window->message_area_frame), cheese_window->message_area);
-  gtk_widget_show (GTK_WIDGET (cheese_window->message_area));
+  gtk_container_add (GTK_CONTAINER (cheese_window->info_bar_frame), cheese_window->info_bar);
+  gtk_widget_show (GTK_WIDGET (cheese_window->info_bar));
 }
 
 static void
@@ -1563,7 +1550,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
 #endif
 
 
-  cheese_window->message_area = NULL;
+  cheese_window->info_bar = NULL;
 
   builder = gtk_builder_new ();
   gtk_builder_add_from_file (builder, PACKAGE_DATADIR "/cheese.ui", &error);
@@ -1594,7 +1581,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->countdown_frame             = GTK_WIDGET (gtk_builder_get_object (builder, "countdown_frame"));
   cheese_window->effect_frame                = GTK_WIDGET (gtk_builder_get_object (builder, "effect_frame"));
   cheese_window->effect_alignment            = GTK_WIDGET (gtk_builder_get_object (builder, "effect_alignment"));
-  cheese_window->message_area_frame          = GTK_WIDGET (gtk_builder_get_object (builder, "message_area_frame"));
+  cheese_window->info_bar_frame              = GTK_WIDGET (gtk_builder_get_object (builder, "info_bar_frame"));
   cheese_window->fullscreen_popup            = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_popup"));
   cheese_window->fullscreen_bar              = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_notebook_bar"));
   cheese_window->button_effects_fullscreen   = GTK_WIDGET (gtk_builder_get_object (builder, "button_effects_fullscreen"));
@@ -1663,6 +1650,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->thumb_view = cheese_thumb_view_new ();
   cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, TRUE);
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_nav);
+
   /* show the scroll window to get it included in the size requisition done later */
   gtk_widget_show_all (cheese_window->thumb_scrollwindow);
 
@@ -1685,7 +1673,14 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_container_add (GTK_CONTAINER (cheese_window->countdown_frame_fullscreen), cheese_window->countdown_fullscreen);
 
   gtk_widget_realize (cheese_window->screen);
-  gdk_window_set_back_pixmap (cheese_window->screen->window, NULL, FALSE);
+  GdkWindow *win = gtk_widget_get_window (cheese_window->screen);
+  if (!gdk_window_ensure_native (win)) {
+    /* FIXME: this breaks offscreen stuff, we should really find
+     * another way to embed video that doesn't require an XID */
+    /* abort: no native window, no xoverlay, no cheese. */
+    g_error ("Could not create a native X11 window for the drawing area");
+  }
+  gdk_window_set_back_pixmap (gtk_widget_get_window (cheese_window->screen), NULL, FALSE);
   gtk_widget_set_app_paintable (cheese_window->screen, TRUE);
   gtk_widget_set_double_buffered (cheese_window->screen, FALSE);
   gtk_widget_add_events (cheese_window->screen, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
@@ -1849,6 +1844,11 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   g_signal_connect (cheese_window->window, "delete_event",
                     G_CALLBACK (cheese_window_delete_event_cb), NULL);
 
+  /* Listen for key presses */
+  gtk_widget_add_events (cheese_window->window, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+  g_signal_connect (cheese_window->window, "key_press_event",
+                    G_CALLBACK (cheese_window_key_press_event_cb), cheese_window);
+
   g_signal_connect (cheese_window->take_picture, "clicked",
                     G_CALLBACK (cheese_window_action_button_clicked_cb), cheese_window);
   g_signal_connect (cheese_window->take_picture_fullscreen, "clicked",
@@ -1869,7 +1869,7 @@ setup_camera (CheeseWindow *cheese_window)
   gdouble    contrast;
   gdouble    saturation;
   gdouble    hue;
-  GtkWidget *message_area;
+  GtkWidget *info_bar;
 
   GError *error;
 
@@ -1943,14 +1943,14 @@ setup_camera (CheeseWindow *cheese_window)
   ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
   if (cheese_webcam_get_num_webcam_devices (cheese_window->webcam) == 0)
   {
-    message_area = cheese_no_camera_message_area ();
+    info_bar = cheese_no_camera_info_bar_new ();
 
-    g_signal_connect (message_area,
+    g_signal_connect (info_bar,
                       "response",
-                      G_CALLBACK (cheese_window_no_camera_message_area_response),
+                      G_CALLBACK (cheese_window_no_camera_info_bar_response),
                       cheese_window);
 
-    cheese_window_set_message_area (cheese_window, message_area);
+    cheese_window_set_info_bar (cheese_window, info_bar);
   }
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), TRUE);
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), TRUE);
@@ -2009,8 +2009,4 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
     g_error_free (error);
     return;
   }
-
-  /* Make URLs and email clickable in about dialog */
-  gtk_about_dialog_set_url_hook (cheese_about_dialog_handle_url, NULL, NULL);
-  gtk_about_dialog_set_email_hook (cheese_about_dialog_handle_email, NULL, NULL);
 }
